___author__ = "Dariusz Dwornikowski"
__copyright__ = "Dariusz Dwornikowski, Poznan University of Technology"
__email__ = "dariusz.dwornikowski@cs.put.poznan.pl"
__version__ = "0.4.6dev"

from pypepa.pepa_model import PEPAModel
